"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const CsvParser = require("papaparse");
const lodash_1 = require("lodash");
const fs_1 = require("fs");
const async_1 = require("async");
const path_1 = require("path");
const constants_1 = require("../ddf-definitions/constants");
const datapackage_1 = require("datapackage");
const db_1 = require("../data/db");
const concept_1 = require("../ddf-definitions/concept");
const file_1 = require("../utils/file");
const shared_1 = require("./shared");
const utils_1 = require("../utils");
const ddf_things_1 = require("../utils/ddf-things");
exports.DATA_PACKAGE_FILE = 'datapackage.json';
exports.stripDdfPrefix = filename => filename.replace(/ddf--(entities|datapoints)--/, '');
const PROCESS_LIMIT = 5;
const LANG_FOLDER = 'lang';
const CSV_EXTENSION = 'csv';
const REQUIRED_DDF_FILE_PARTS = 2;
const getConceptType = (fileParts) => lodash_1.head(fileParts) === 'concepts' ? constants_1.CONCEPT : null;
const getEntityType = (fileParts) => lodash_1.head(fileParts) === 'entities' ? constants_1.ENTITY : null;
const getSynonymType = (fileParts) => lodash_1.head(fileParts) === 'synonyms' ? constants_1.SYNONYM : null;
const getDataPointType = (fileParts) => lodash_1.head(fileParts) === 'datapoints' ? constants_1.DATA_POINT : null;
const getDdfParts = (fileParts) => {
    if (fileParts.length >= REQUIRED_DDF_FILE_PARTS && lodash_1.head(fileParts) === 'ddf') {
        return lodash_1.drop(fileParts);
    }
    return null;
};
const parseDdfFile = (rootFolder, folder, filename) => {
    const partsByPoint = lodash_1.split(filename, '.');
    if (partsByPoint.length <= 1 || partsByPoint[1] !== CSV_EXTENSION) {
        return { valid: false };
    }
    const ddfParts = getDdfParts(lodash_1.split(lodash_1.head(partsByPoint), constants_1.DDF_SEPARATOR));
    if (!ddfParts) {
        return { valid: false };
    }
    const type = getConceptType(ddfParts) ||
        getEntityType(ddfParts) ||
        getSynonymType(ddfParts) ||
        getDataPointType(ddfParts);
    if (!type) {
        return { valid: false };
    }
    const fullPath = path_1.resolve(folder, filename);
    return {
        valid: true,
        filename,
        name: lodash_1.head(partsByPoint),
        fullPath,
        rootFolder,
        type,
        parts: lodash_1.drop(ddfParts)
    };
};
const getActualSubDirectories = (folder, settings, onSubDirsReady) => {
    file_1.walkDir(folder, (err, folders) => {
        if (err) {
            onSubDirsReady(err);
            return;
        }
        const excludeDirs = shared_1.getExcludedDirs(settings);
        const actualFolders = folders.filter(folder => shared_1.isPathExpected(folder, excludeDirs));
        actualFolders.push(folder);
        onSubDirsReady(null, actualFolders);
    });
};
let directoryIndex = 0;
let currentDir = null;
let prevDir = null;
class DataPackage {
    constructor(rootFolder, settings) {
        this.isNewDataPackage = false;
        this.rootFolder = rootFolder;
        this.settings = settings;
        this.errors = [];
        this.warnings = [];
        this.fileDescriptors = [];
        this.dataPackageContent = {};
        directoryIndex = 0;
        currentDir = null;
        prevDir = null;
    }
    getTranslationFileDescriptors(onTranslationsFileDescriptorsReady) {
        file_1.readDir(path_1.resolve(this.rootFolder, LANG_FOLDER), (errFolders, translationFolders) => {
            if (errFolders) {
                this.warnings.push({
                    source: errFolders,
                    reason: 'translation folder reading'
                });
                onTranslationsFileDescriptorsReady();
                return;
            }
            const translationsFolderActions = translationFolders.map(translationFolder => (onTranslationFolderReady) => {
                const translationFullFolder = path_1.resolve(this.rootFolder, LANG_FOLDER, translationFolder);
                this.getDdfFileDescriptors(translationFullFolder, this.settings, (folderErr, ddfTransFileDescriptors = []) => {
                    const transFileDescriptors = ddfTransFileDescriptors
                        .map(transFileDescriptor => {
                        transFileDescriptor.translation = true;
                        return transFileDescriptor;
                    });
                    onTranslationFolderReady(folderErr, transFileDescriptors);
                });
            });
            async_1.parallelLimit(translationsFolderActions, PROCESS_LIMIT, (err, translationsData) => {
                onTranslationsFileDescriptorsReady(err, translationFolders, lodash_1.flatten(translationsData, true));
            });
        });
    }
    fillConstraints(ddfFileDescriptor) {
        if (ddfFileDescriptor.valid && ddfFileDescriptor.type === constants_1.DATA_POINT) {
            const constraintsProcessing = (part) => {
                const details = lodash_1.compact(lodash_1.split(part, '-'));
                if (details.length <= 1) {
                    ddfFileDescriptor.valid = false;
                    return null;
                }
                const correctedPart = lodash_1.head(details);
                ddfFileDescriptor.constraints[correctedPart] = lodash_1.tail(details);
                return correctedPart;
            };
            let isByWordPassed = false;
            ddfFileDescriptor.constraints = {};
            ddfFileDescriptor.parts.forEach((part, index) => {
                if (isByWordPassed && lodash_1.includes(part, '-')) {
                    const correctedPart = constraintsProcessing(part);
                    if (correctedPart) {
                        ddfFileDescriptor.parts[index] = correctedPart;
                    }
                }
                if (part === 'by') {
                    isByWordPassed = true;
                }
            });
        }
        return ddfFileDescriptor;
    }
    getDdfFileDescriptors(folder, settings, onDdfFileDescriptorsReady) {
        getActualSubDirectories(folder, settings, (dirErr, dirs) => {
            if (dirErr) {
                onDdfFileDescriptorsReady(dirErr);
                return;
            }
            const actions = dirs.map(dir => onDirRead => {
                file_1.readDir(dir, (err, files = []) => {
                    const ddfFileDescriptors = files
                        .map(file => this.fillConstraints(parseDdfFile(this.rootFolder, dir, file)))
                        .filter(ddfFile => ddfFile.type);
                    onDirRead(err, ddfFileDescriptors);
                });
            });
            async_1.parallelLimit(actions, PROCESS_LIMIT, (err, ddfFileDescriptors) => {
                const _ddfFileDescriptors = lodash_1.compact(lodash_1.flatten(ddfFileDescriptors));
                for (let ddfFileDescriptor of _ddfFileDescriptors) {
                    currentDir = path.dirname(ddfFileDescriptor.fullPath);
                    if (directoryIndex === 0 || currentDir !== prevDir) {
                        directoryIndex++;
                    }
                    prevDir = currentDir;
                    ddfFileDescriptor.directoryIndex = directoryIndex;
                }
                onDdfFileDescriptorsReady(err, _ddfFileDescriptors);
            });
        });
    }
    fillHeaders(onHeadersReady) {
        const headerGetActions = this.fileDescriptors.map((fileDescriptor) => onHeaderReady => {
            file_1.getFileLine(fileDescriptor.fullPath, 0, (err, firstLine) => {
                fileDescriptor.headers = [];
                if (err) {
                    return onHeaderReady();
                }
                const parsedFirstCsvLine = CsvParser.parse(firstLine);
                fileDescriptor.headers = lodash_1.head(parsedFirstCsvLine.data);
                onHeaderReady();
            });
        });
        async_1.parallelLimit(headerGetActions, PROCESS_LIMIT, onHeadersReady);
    }
    fillPrimaryKeys(conceptTypeHash, conceptDomainHash) {
        const normalizePrimaryKey = (fileDescriptor) => {
            if (typeof fileDescriptor.primaryKey === 'string') {
                return null;
            }
            return fileDescriptor.primaryKey.map(pkPart => {
                if (!lodash_1.includes(fileDescriptor.headers, pkPart)) {
                    return conceptDomainHash[pkPart];
                }
                return pkPart;
            });
        };
        this.fileDescriptors.forEach(fileDescriptor => {
            if (fileDescriptor.type === constants_1.CONCEPT) {
                fileDescriptor.primaryKey = constants_1.CONCEPT_ID;
            }
            if (fileDescriptor.type === constants_1.ENTITY) {
                const [domain, set] = fileDescriptor.parts;
                fileDescriptor.primaryKey = fileDescriptor.headers.indexOf(set) > -1 ? set : domain;
            }
            if (fileDescriptor.type === constants_1.SYNONYM) {
                const entityDomain = fileDescriptor.headers.find(header => conceptTypeHash[header] === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN);
                const entitySet = fileDescriptor.headers.find(header => conceptTypeHash[header] === ddf_things_1.CONCEPT_TYPE_ENTITY_SET);
                fileDescriptor.primaryKey = [entityDomain || entitySet || constants_1.CONCEPT_ID, constants_1.SYNONYM_ID];
            }
            if (fileDescriptor.type === constants_1.DATA_POINT) {
                const ddfDataPointSeparatorPos = lodash_1.indexOf(fileDescriptor.parts, constants_1.DDF_DATAPOINT_SEPARATOR);
                const primaryKeyPartsCount = fileDescriptor.parts.length - ddfDataPointSeparatorPos - 1;
                fileDescriptor.primaryKey = lodash_1.takeRight(fileDescriptor.parts, primaryKeyPartsCount);
                fileDescriptor.primaryKey = normalizePrimaryKey(fileDescriptor);
            }
        });
    }
    getDataPackageObject() {
        if (!this.isNewDataPackage) {
            return this.dataPackageContent;
        }
        const rootFolderParts = lodash_1.split(this.rootFolder, path_1.sep);
        const packageName = lodash_1.last(rootFolderParts);
        const prepareField = (header, fileDescriptor) => {
            const result = { name: header };
            if (fileDescriptor.constraints && fileDescriptor.constraints[header]) {
                result.constraints = { enum: fileDescriptor.constraints[header] };
            }
            return result;
        };
        const getRelativeDir = (fullPath) => {
            const relativeDir = path.relative(this.rootFolder, path.parse(fullPath).dir);
            return lodash_1.isEmpty(relativeDir) ? '' : `${relativeDir}/`;
        };
        const getNameSuffix = currentDirectoryIndex => currentDirectoryIndex === 1 ? '' : `-${currentDirectoryIndex}`;
        return {
            name: packageName || 'new_dataset',
            title: packageName || 'New Dataset',
            description: '',
            version: '0.0.1',
            language: { id: 'en', name: 'English' },
            translations: this.translationFolders
                .map(translation => ({ id: translation, name: translation })),
            license: '',
            author: '',
            resources: this.fileDescriptors
                .map((fileDescriptor) => ({
                path: `${getRelativeDir(fileDescriptor.fullPath)}${fileDescriptor.filename}`,
                name: `${exports.stripDdfPrefix(fileDescriptor.name)}${getNameSuffix(fileDescriptor.directoryIndex)}`,
                schema: {
                    fields: (fileDescriptor.headers || []).map(header => prepareField(header, fileDescriptor)),
                    primaryKey: fileDescriptor.primaryKey
                }
            })),
            ddfSchema: this.dataPackageContent.ddfSchema
        };
    }
    buildDataPackageResources(onDataPackageReady) {
        this.db = new db_1.Db();
        this.errors = [];
        this.warnings = [];
        this.fileDescriptors = [];
        this.getDdfFileDescriptors(this.rootFolder, this.settings, (ddfFileErr, ddfFileDescriptors) => {
            if (ddfFileErr) {
                this.errors.push({
                    source: ddfFileErr,
                    reason: 'ddf files reading'
                });
                onDataPackageReady();
                return;
            }
            this.getTranslationFileDescriptors((translationsErr, translationFolders) => {
                this.warnings.push({
                    source: translationsErr,
                    reason: 'translation file reading'
                });
                this.fileDescriptors = lodash_1.compact(ddfFileDescriptors);
                this.translationFolders = translationFolders || [];
                this.fillHeaders(headersError => {
                    if (headersError) {
                        this.errors.push({ source: headersError, reason: 'headers reading' });
                    }
                    const concept = new concept_1.Concept(this.db);
                    const conceptLoadingActions = this.fileDescriptors
                        .filter(fileDescriptor => fileDescriptor.type === constants_1.CONCEPT)
                        .map(fileDescriptor => {
                        return onFileLoaded => {
                            this.db.fillCollection(Symbol.keyFor(fileDescriptor.type), fileDescriptor.fullPath, onFileLoaded, false);
                        };
                    });
                    async_1.parallelLimit(conceptLoadingActions, PROCESS_LIMIT, conceptsErr => {
                        if (conceptsErr) {
                            this.errors.push({ source: conceptsErr, reason: 'concepts reading' });
                        }
                        const conceptTypeHash = concept.getDictionary(null, constants_1.CONCEPT_TYPE);
                        const conceptDomainHash = concept.getDictionary(null, constants_1.DOMAIN_ID);
                        this.fillPrimaryKeys(conceptTypeHash, conceptDomainHash);
                        datapackage_1.validate(this.getDataPackageObject()).then(consistency => {
                            this.consistencyDescriptor = consistency;
                            if (!this.consistencyDescriptor.valid) {
                                return onDataPackageReady();
                            }
                            onDataPackageReady(this.getDataPackageObject());
                        }).catch(err => {
                            this.errors.push({
                                source: err.message,
                                reason: `${exports.DATA_PACKAGE_FILE} frictionless validating`
                            });
                            onDataPackageReady();
                        });
                    });
                });
            });
        });
    }
    read(onDataPackageReady) {
        this.errors = [];
        this.warnings = [];
        this.fileDescriptors = [];
        try {
            const content = fs_1.readFileSync(path_1.resolve(this.rootFolder, exports.DATA_PACKAGE_FILE), 'utf8');
            this.dataPackageContent = JSON.parse(content);
        }
        catch (contentErr) {
            this.errors.push({
                source: contentErr,
                reason: `${exports.DATA_PACKAGE_FILE} parsing`
            });
            onDataPackageReady();
        }
        this.getTranslationFileDescriptors((translationsErr, translationFolders) => {
            this.warnings.push({
                source: translationsErr,
                reason: 'translation file reading'
            });
            this.translationFolders = translationFolders || [];
            onDataPackageReady(this.dataPackageContent);
        });
    }
    setSchema(ddfSchema) {
        this.dataPackageContent.ddfSchema = ddfSchema;
    }
    getTranslations() {
        return this.dataPackageContent.translations || [];
    }
    take(onDataPackageReady, ignoreExistingDataPackage) {
        const filePath = path_1.resolve(this.rootFolder, exports.DATA_PACKAGE_FILE);
        this.isNewDataPackage = false;
        file_1.fileExists(filePath, (err, isExists) => {
            if (err || !isExists || ignoreExistingDataPackage) {
                if (err) {
                    utils_1.logger.error(err);
                }
                this.isNewDataPackage = true;
                this.buildDataPackageResources(dataPackage => onDataPackageReady(dataPackage));
            }
            else {
                this.read(dataPackage => {
                    this.getDdfFileDescriptors(this.rootFolder, this.settings, (ddfFileErr, ddfFileDescriptors) => {
                        if (ddfFileErr) {
                            this.errors.push({
                                source: ddfFileErr,
                                reason: `${exports.DATA_PACKAGE_FILE} processing`
                            });
                            return onDataPackageReady(dataPackage);
                        }
                        this.getTranslationFileDescriptors((translationsErr, translationFolders) => {
                            this.warnings.push({
                                source: translationsErr,
                                reason: 'translation file reading'
                            });
                            this.fileDescriptors = lodash_1.compact(ddfFileDescriptors);
                            this.dataPackageContent = dataPackage;
                            this.translationFolders = translationFolders || [];
                            this.fillHeaders(headersError => {
                                if (headersError) {
                                    this.errors.push({ source: headersError, reason: 'headers reading' });
                                }
                                datapackage_1.validate(this.getDataPackageObject()).then(consistency => {
                                    this.consistencyDescriptor = consistency;
                                    if (!this.consistencyDescriptor.valid) {
                                        return onDataPackageReady();
                                    }
                                    onDataPackageReady(this.getDataPackageObject());
                                }).catch(err => {
                                    this.errors.push({
                                        source: err.message,
                                        reason: `${exports.DATA_PACKAGE_FILE} frictionless validation`
                                    });
                                    onDataPackageReady(dataPackage);
                                });
                            });
                        });
                    });
                });
            }
        });
    }
    isValid() {
        return lodash_1.isEmpty(this.errors);
    }
}
exports.DataPackage = DataPackage;
