"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const shared_1 = require("./shared");
const lodash_1 = require("lodash");
const async_1 = require("async");
const path_1 = require("path");
const constants_1 = require("../ddf-definitions/constants");
const data_package_1 = require("./data-package");
const file_descriptor_1 = require("./file-descriptor");
const utils_1 = require("../utils");
const PROCESS_LIMIT = 30;
class DDFRoot {
    constructor(path, settings, ignoreExistingDataPackage = false) {
        this.path = path;
        this.settings = settings || {};
        this.settings.excludeDirs = shared_1.getExcludedDirs(this.settings);
        this.errors = [];
        this.ignoreExistingDataPackage = ignoreExistingDataPackage;
        this.isEmpty = false;
        this.isDDF = true;
        this.fileDescriptors = [];
    }
    getTranslations(onTranslationsReady) {
        if (lodash_1.isEmpty(this.dataPackageDescriptor.getTranslations())) {
            onTranslationsReady();
            return;
        }
        const translationsIds = this.dataPackageDescriptor.getTranslations().map(translation => translation.id);
        this.fileDescriptors.forEach(fileDescriptor => {
            const translationFolder = path_1.resolve(this.path, constants_1.TRNSLATIONS_FOLDER);
            fileDescriptor.transFileDescriptors = translationsIds
                .map(translationId => new file_descriptor_1.FileDescriptor({
                dir: path_1.resolve(translationFolder, translationId),
                file: fileDescriptor.file,
                type: fileDescriptor.type,
                primaryKey: fileDescriptor.primaryKey,
                fullPath: path_1.resolve(translationFolder, translationId, fileDescriptor.file),
                translationId,
                isTranslation: true
            }));
        });
        const transFileActions = this.fileDescriptors
            .map(fileDescriptor => onTransFileReady => fileDescriptor.checkTranslations(onTransFileReady));
        async_1.parallelLimit(transFileActions, PROCESS_LIMIT, onTranslationsReady);
    }
    check(onDirectoryDescriptorReady) {
        this.dataPackageDescriptor = new data_package_1.DataPackage(this.path, this.settings);
        this.dataPackageDescriptor.take(dataPackageObject => {
            if (!this.dataPackageDescriptor.isValid() || lodash_1.isEmpty(this.dataPackageDescriptor.fileDescriptors)) {
                this.isDDF = false;
                onDirectoryDescriptorReady();
                return;
            }
            const expectedResources = !this.settings.datapointlessMode ? dataPackageObject.resources :
                dataPackageObject.resources.filter(ddfResource => !lodash_1.isArray(ddfResource.schema.primaryKey) ||
                    lodash_1.includes(ddfResource.schema.primaryKey, data_package_1.SYNONYM_ID));
            this.fileDescriptors = expectedResources.map(ddfResource => this.getFileDescriptor(this.path, ddfResource));
            utils_1.logger.progressInit('root checking', { total: this.fileDescriptors.length });
            const actions = this.fileDescriptors.map(fileDescriptor => onFileChecked => fileDescriptor.check(onFileChecked));
            async_1.parallelLimit(actions, PROCESS_LIMIT, checkErr => {
                if (checkErr) {
                    onDirectoryDescriptorReady(checkErr);
                    return;
                }
                this.getTranslations(onDirectoryDescriptorReady);
            });
        }, this.ignoreExistingDataPackage);
    }
    getFileDescriptor(dir, ddfResource) {
        return new file_descriptor_1.FileDescriptor({
            dir,
            file: ddfResource.path,
            type: this.dataPackageDescriptor.getType(ddfResource.path),
            headers: ddfResource.schema.fields,
            primaryKey: ddfResource.schema.primaryKey,
            fullPath: path_1.resolve(dir, ddfResource.path)
        });
    }
    getDataPackageResources() {
        return this.dataPackageDescriptor.getResources();
    }
    getDataPackageSchema() {
        return this.dataPackageDescriptor.getSchema();
    }
}
exports.DDFRoot = DDFRoot;
