"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const async_1 = require("async");
const lodash_1 = require("lodash");
const file_1 = require("../utils/file");
const shared_1 = require("./shared");
const ddf_things_1 = require("../utils/ddf-things");
const constants_1 = require("../ddf-definitions/constants");
const args_1 = require("../utils/args");
const settings = args_1.getSettings();
function recursivePermutation(pkSets) {
    if (pkSets.length === 0) {
        return [[]];
    }
    let pkSet = pkSets.pop();
    let prevPermutations = recursivePermutation(pkSets);
    let allPermutations = [];
    for (let pkConcept of pkSet) {
        for (let prevPerm of prevPermutations) {
            let newPermutation = prevPerm.slice();
            newPermutation.push(pkConcept);
            allPermutations.push(newPermutation);
        }
    }
    return allPermutations;
}
function addToSchema(schema, resourceSchema) {
    const key = resourceSchema.primaryKey.sort().join('-');
    const hash = key + '--' + (resourceSchema.value || '-null-');
    if (!schema[hash]) {
        schema[hash] = {
            primaryKey: resourceSchema.primaryKey,
            value: resourceSchema.value,
            resources: new Set()
        };
    }
    schema[hash].resources.add(resourceSchema.resource);
}
function getDdfSchemaContent(dataset, onDdfSchemaReady) {
    const entityConcepts = {};
    const conceptsContent = dataset.ddfDataSet.getConcept().getDataByFiles();
    const conceptsFiles = lodash_1.keys(conceptsContent);
    const entitiesContent = dataset.ddfDataSet.getEntity().getDataByFiles();
    const entitiesFiles = lodash_1.keys(entitiesContent);
    const synonymsContent = dataset.ddfDataSet.getSynonym().getDataByFiles();
    const synonymsFiles = lodash_1.keys(synonymsContent);
    const getProgressBar = (config) => settings.isDataPackageGenerationMode && !settings.silent ?
        require('terminal-kit').terminal.progressBar(config) : {
        startItem: () => {
        },
        itemDone: () => {
        }
    };
    dataset.dataHash = {};
    for (let conceptFile of conceptsFiles) {
        const relativePath = shared_1.getRelativePath(conceptFile, dataset.dataPackageDescriptor.rootFolder);
        dataset.dataHash[relativePath] = conceptsContent[conceptFile];
    }
    for (let synonymFile of synonymsFiles) {
        const relativePath = shared_1.getRelativePath(synonymFile, dataset.dataPackageDescriptor.rootFolder);
        dataset.dataHash[relativePath] = synonymsContent[synonymFile];
    }
    for (let entityFile of entitiesFiles) {
        const relativePath = shared_1.getRelativePath(entityFile, dataset.dataPackageDescriptor.rootFolder);
        dataset.dataHash[relativePath] = entitiesContent[entityFile];
    }
    for (let resource of dataset.conceptsResources) {
        if (!dataset.dataHash[resource.path]) {
            return onDdfSchemaReady(new Error(`ddfSchema creation error: key=${resource.path}`));
        }
        for (let row of dataset.dataHash[resource.path]) {
            if (row.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN || row.concept_type === ddf_things_1.CONCEPT_TYPE_ENTITY_SET) {
                entityConcepts[row.concept] = row;
            }
        }
    }
    const entities = {};
    for (let resource of dataset.entitiesResources) {
        const pk = resource.schema.primaryKey[0];
        if (!entityConcepts[pk]) {
            return onDdfSchemaReady(`Validator internal error: entity concept not found`);
        }
        const domain = (entityConcepts[pk].concept_type == ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN) ? pk : entityConcepts[pk].domain;
        const entity_set_fields = [];
        for (let field of resource.schema['fields']) {
            if (ddf_things_1.looksLikeIsField(field)) {
                entity_set_fields.push(field);
            }
        }
        if (!dataset.dataHash[resource.path]) {
            return onDdfSchemaReady(new Error(`ddfSchema creation error: key=${resource.path}`));
        }
        for (let row of dataset.dataHash[resource.path]) {
            if (!entities[domain]) {
                entities[domain] = {};
            }
            if (!entities[domain][row[pk]]) {
                entities[domain][row[pk]] = new Set();
            }
            entities[domain][row[pk]].add(domain);
            for (let field of entity_set_fields) {
                if (ddf_things_1.isDdfTrue(row[field])) {
                    entities[domain][row[pk]].add(field.substring(4));
                }
            }
        }
    }
    const schema = {};
    const tasks = dataset.resources.map(resource => resource.name);
    const progressBar = getProgressBar({
        width: 80,
        title: 'resources hash processing:',
        eta: true,
        percent: true,
        items: tasks.length
    });
    const actions = dataset.resources.map(resource => onResourceProcessed => {
        const task = tasks.shift();
        progressBar.startItem(task);
        file_1.readFile(path.resolve(dataset.dataPackageDescriptor.rootFolder, resource.path), (err, dataPointContent) => {
            if (err) {
                onResourceProcessed(err);
                return;
            }
            let primaryKeyEntityConcepts = [];
            let primaryKeySetsStatic = [];
            for (let pkField of resource.schema.primaryKey) {
                if (entityConcepts[pkField]) {
                    const domain = entityConcepts[pkField].concept_type == ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN ? pkField : entityConcepts[pkField].domain;
                    primaryKeyEntityConcepts.push({ concept: pkField, domain: domain });
                }
                else {
                    primaryKeySetsStatic.push(new Set([pkField]));
                }
            }
            if (primaryKeyEntityConcepts.length == 0) {
                if (resource.schema.primaryKey.length == resource.schema.fields.length) {
                    addToSchema(schema, {
                        primaryKey: resource.schema.primaryKey,
                        value: null,
                        resource: resource.name
                    });
                }
                else {
                    for (let field of resource.schema.fields) {
                        if (!lodash_1.includes(resource.schema.primaryKey, field)) {
                            addToSchema(schema, {
                                primaryKey: resource.schema.primaryKey,
                                value: field,
                                resource: resource.name
                            });
                        }
                    }
                }
            }
            else {
                for (let row of dataPointContent) {
                    const primaryKeySets = primaryKeySetsStatic.slice();
                    for (const pkEntityConcept of primaryKeyEntityConcepts) {
                        primaryKeySets.push(entities[pkEntityConcept.domain][row[pkEntityConcept.concept]]);
                    }
                    const primaryKeyPermutations = recursivePermutation(lodash_1.compact(primaryKeySets));
                    if (resource.schema.primaryKey.length == resource.schema.fields.length) {
                        for (let pkPerm of primaryKeyPermutations) {
                            addToSchema(schema, {
                                primaryKey: pkPerm,
                                value: null,
                                resource: resource.name
                            });
                        }
                    }
                    else {
                        for (let field of resource.schema.fields) {
                            if (!resource.schema.primaryKey.includes(field))
                                for (let pkPerm of primaryKeyPermutations) {
                                    addToSchema(schema, {
                                        primaryKey: pkPerm,
                                        value: field,
                                        resource: resource.name
                                    });
                                }
                        }
                    }
                }
            }
            progressBar.itemDone();
            onResourceProcessed();
        });
    });
    async_1.parallelLimit(actions, 10, (err) => {
        const ddfSchema = { datapoints: [], entities: [], concepts: [], synonyms: [] };
        for (let key of lodash_1.keys(schema)) {
            const keyValueObject = schema[key];
            const type = constants_1.getTypeByPrimaryKey(keyValueObject.primaryKey);
            keyValueObject.resources = Array.from(keyValueObject.resources);
            switch (type) {
                case constants_1.CONCEPT:
                    ddfSchema.concepts.push(keyValueObject);
                    break;
                case constants_1.ENTITY:
                    ddfSchema.entities.push(keyValueObject);
                    break;
                case constants_1.SYNONYM:
                    ddfSchema.synonyms.push(keyValueObject);
                    break;
                case constants_1.DATA_POINT:
                    ddfSchema.datapoints.push(keyValueObject);
                    break;
                default:
                    break;
            }
        }
        onDdfSchemaReady(err, ddfSchema);
    });
}
exports.getDdfSchemaContent = getDdfSchemaContent;
