"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const Papa = require("papaparse");
const async_1 = require("async");
const lodash_1 = require("lodash");
const fs_1 = require("fs");
const file_1 = require("../utils/file");
const registry_1 = require("../ddf-rules/registry");
const csv_checker_1 = require("./csv-checker");
const utils_1 = require("../utils");
const PROCESS_LIMIT = 10;
class FileDescriptor {
    constructor(data) {
        this.dir = data.dir;
        this.file = data.file;
        this.type = data.type;
        this.primaryKey = data.primaryKey;
        this.headers = this.issues = this.transFileDescriptors = this.content = [];
        this.measures = data.measures;
        this.fullPath = data.fullPath;
        this.csvChecker = new csv_checker_1.CsvChecker(this.fullPath);
        this.hasFirstLine = false;
        this.translationId = data.translationId;
        this.isTranslation = data.isTranslation;
    }
    fillHeaders(onHeadersReady) {
        file_1.getFileLine(this.fullPath, 0, (err, line) => {
            if (err) {
                onHeadersReady(err);
                return;
            }
            this.headers = Papa.parse(line, { header: true }).meta.fields;
            onHeadersReady();
        });
    }
    is(type) {
        if (!lodash_1.isArray(type)) {
            return type === this.type;
        }
        return lodash_1.includes(type, this.type);
    }
    check(onFileDescriptorChecked) {
        this.checkFile((fileStateDescriptor) => {
            if (!fileStateDescriptor.ok) {
                this.issues.push(fileStateDescriptor);
                return onFileDescriptorChecked([fileStateDescriptor]);
            }
            file_1.getFileLine(this.fullPath, 1, (lineErr, line) => {
                if (lineErr) {
                    utils_1.logger.error(lineErr);
                }
                const fileAttributesContainer = fileStateDescriptor;
                this.hasFirstLine = !lineErr && !!line;
                if (fileAttributesContainer && fileAttributesContainer.stats) {
                    this.size = fileAttributesContainer.stats.size;
                }
                this.csvChecker.check(() => {
                    utils_1.logger.progress();
                    return onFileDescriptorChecked();
                });
                utils_1.logger.progress();
            });
        });
    }
    checkTranslations(onTranslationsChecked) {
        const transFileActions = this.transFileDescriptors.map(transFileDescriptor => onTransFileReady => {
            transFileDescriptor.check(err => {
                if (err) {
                    onTransFileReady();
                    return;
                }
                transFileDescriptor.fillHeaders(onTransFileReady);
            });
        });
        async_1.parallelLimit(transFileActions, PROCESS_LIMIT, onTranslationsChecked);
    }
    getExistingTranslationDescriptors() {
        return this.transFileDescriptors.filter(transFileDescriptor => lodash_1.isEmpty(transFileDescriptor.issues));
    }
    checkFile(onFileChecked) {
        fs_1.stat(this.fullPath, (err, stats) => {
            if (err) {
                onFileChecked({
                    type: registry_1.INCORRECT_FILE,
                    fullPath: this.fullPath,
                    data: err.message
                });
                return;
            }
            if (!stats.isFile()) {
                onFileChecked({
                    type: registry_1.INCORRECT_FILE,
                    fullPath: this.fullPath,
                    data: `${this.fullPath} is not a file`
                });
                return;
            }
            onFileChecked({ ok: true, stats });
        });
    }
}
exports.FileDescriptor = FileDescriptor;
