"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const async_1 = require("async");
const lodash_1 = require("lodash");
const constants_1 = require("./constants");
const concept_1 = require("./concept");
const entity_1 = require("./entity");
const synonym_1 = require("./synonym");
const data_point_1 = require("./data-point");
const db_1 = require("../data/db");
const data_package_1 = require("../data/data-package");
const utils_1 = require("../utils");
const shared_1 = require("../shared");
const ddf_things_1 = require("../utils/ddf-things");
const file_descriptor_1 = require("../data/file-descriptor");
const shared_2 = require("../data/shared");
const path_1 = require("path");
const ddf_schema_1 = require("../data/ddf-schema");
const file_1 = require("../utils/file");
const PROCESS_LIMIT = 30;
class DdfDataSet {
    constructor(rootPath, settings, ignoreExistingDataPackage = false) {
        this.rootPath = rootPath;
        this.db = new db_1.Db();
        this.settings = settings || {};
        this.settings.excludeDirs = shared_2.getExcludedDirs(this.settings);
        this.errors = [];
        this.ignoreExistingDataPackage = ignoreExistingDataPackage;
        this.isEmpty = false;
        this.isDDF = true;
        this.fileDescriptors = [];
        this.wrongFileDescriptors = [];
        this.expectedClass = {
            [constants_1.CONCEPT]: new concept_1.Concept(this.db),
            [constants_1.ENTITY]: new entity_1.Entity(this.db),
            [constants_1.SYNONYM]: new synonym_1.Synonym(this.db),
            [constants_1.DATA_POINT]: new data_point_1.DataPoint()
        };
    }
    load(onDataSetLoaded) {
        this.check(() => {
            const loaders = [];
            if (this.isDDF) {
                const expectedFileDescriptors = this.fileDescriptors.filter(fileDescriptor => lodash_1.isEmpty(fileDescriptor.issues));
                this.wrongFileDescriptors = this.fileDescriptors.filter(fileDescriptor => !lodash_1.isEmpty(fileDescriptor.issues));
                utils_1.logger.progressInit('dataset loading', { total: expectedFileDescriptors.length });
                expectedFileDescriptors.forEach(fileDescriptor => {
                    if (fileDescriptor.is(constants_1.DATA_POINT) && fileDescriptor.csvChecker.isCorrect()) {
                        loaders.push(onFileLoaded => {
                            fileDescriptor.fillHeaders(() => {
                                this.expectedClass[fileDescriptor.type].addDescriptors(fileDescriptor, this.dataPackageDescriptor);
                                utils_1.logger.progress();
                                onFileLoaded();
                            });
                        });
                    }
                    if (fileDescriptor.is([constants_1.CONCEPT, constants_1.ENTITY, constants_1.SYNONYM]) && fileDescriptor.csvChecker.isCorrect()) {
                        loaders.push(onFileLoaded => {
                            fileDescriptor.fillHeaders(() => {
                                this.db.fillCollection(Symbol.keyFor(fileDescriptor.type), fileDescriptor.fullPath, fileErr => {
                                    this.expectedClass[fileDescriptor.type].addFileDescriptor(fileDescriptor);
                                    if (this.expectedClass[fileDescriptor.type].getTranslationsData) {
                                        this.expectedClass[fileDescriptor.type]
                                            .getTranslationsData(translationsErr => onFileLoaded(fileErr || translationsErr));
                                    }
                                    else {
                                        onFileLoaded(fileErr);
                                    }
                                    utils_1.logger.progress();
                                }, false);
                            });
                        });
                    }
                });
            }
            async_1.parallelLimit(loaders, 10, (err, definitions) => {
                if (shared_1.supervisor.abandon) {
                    return onDataSetLoaded(new Error('abandoned by external reason'));
                }
                const allMeasures = this.getAllMeasures();
                this.definitions = lodash_1.compact(definitions);
                this.fileDescriptors.forEach(fileDescriptor => {
                    fileDescriptor.measures = lodash_1.intersection(allMeasures, fileDescriptor.headers);
                });
                onDataSetLoaded(err);
            });
        });
    }
    getConcept() {
        return this.expectedClass[constants_1.CONCEPT];
    }
    getEntity() {
        return this.expectedClass[constants_1.ENTITY];
    }
    getSynonym() {
        return this.expectedClass[constants_1.SYNONYM];
    }
    getDataPoint() {
        return this.expectedClass[constants_1.DATA_POINT];
    }
    getAllMeasures() {
        return this.getConcept().getAllData()
            .filter(record => record.concept_type === ddf_things_1.CONCEPT_TYPE_MEASURE)
            .map(record => record.concept);
    }
    getConceptsByType(...types) {
        return this.getConcept().getAllData()
            .filter(record => lodash_1.includes(types, record.concept_type))
            .map(record => record.concept);
    }
    getDataPackageDescriptor() {
        return this.dataPackageDescriptor;
    }
    getTranslations(onTranslationsReady) {
        if (lodash_1.isEmpty(this.dataPackageDescriptor.getTranslations())) {
            onTranslationsReady();
            return;
        }
        const translationsIds = this.dataPackageDescriptor.getTranslations().map(translation => translation.id);
        this.fileDescriptors.forEach(fileDescriptor => {
            const translationFolder = path_1.resolve(this.rootPath, constants_1.TRANSLATIONS_FOLDER);
            fileDescriptor.transFileDescriptors = translationsIds
                .map(translationId => new file_descriptor_1.FileDescriptor({
                dir: path_1.resolve(translationFolder, translationId),
                file: fileDescriptor.file,
                type: fileDescriptor.type,
                primaryKey: fileDescriptor.primaryKey,
                fullPath: path_1.resolve(translationFolder, translationId, fileDescriptor.file),
                translationId,
                isTranslation: true
            }));
        });
        const transFileActions = this.fileDescriptors
            .map(fileDescriptor => onTransFileReady => fileDescriptor.checkTranslations(onTransFileReady));
        async_1.parallelLimit(transFileActions, PROCESS_LIMIT, onTranslationsReady);
    }
    check(onDirectoryDescriptorReady) {
        const process = (dataPackageObject) => {
            if (!this.dataPackageDescriptor.isValid()) {
                this.isDDF = false;
                onDirectoryDescriptorReady();
                return;
            }
            const dp = constants_1.getDecoratedDataPackageObject(dataPackageObject);
            const expectedResources = dp.getAllResources();
            this.fileDescriptors = expectedResources.map(ddfResource => this.getFileDescriptor(this.rootPath, ddfResource));
            utils_1.logger.progressInit('root checking', { total: this.fileDescriptors.length });
            const actions = this.fileDescriptors.map(fileDescriptor => onFileChecked => fileDescriptor.check(onFileChecked));
            async_1.parallelLimit(actions, PROCESS_LIMIT, checkErr => {
                if (checkErr) {
                    onDirectoryDescriptorReady(checkErr);
                    return;
                }
                this.getTranslations(onDirectoryDescriptorReady);
            });
        };
        this.dataPackageDescriptor = new data_package_1.DataPackage(this.rootPath, this.settings);
        this.dataPackageDescriptor.take(() => {
            process(this.dataPackageDescriptor.getDataPackageObject());
        }, this.ignoreExistingDataPackage);
    }
    createDataPackage(onDataPackageCreated) {
        this.load(loadErr => {
            if (loadErr) {
                return onDataPackageCreated(loadErr);
            }
            this.fillDdfSchema((schemaErr, ddfSchema) => {
                this.dataPackageDescriptor.setSchema(ddfSchema);
                onDataPackageCreated(schemaErr);
            });
        });
    }
    getFileDescriptor(dir, ddfResource) {
        return new file_descriptor_1.FileDescriptor({
            dir,
            file: ddfResource.path,
            type: constants_1.getTypeByPrimaryKey(ddfResource.schema.primaryKey),
            headers: ddfResource.schema.fields,
            primaryKey: ddfResource.schema.primaryKey,
            fullPath: path_1.resolve(dir, ddfResource.path)
        });
    }
    getDataPackageResources() {
        return this.dataPackageDescriptor.getDataPackageObject().resources;
    }
    getDataPackageSchema() {
        return this.dataPackageDescriptor.getDataPackageObject().ddfSchema;
    }
    fillDdfSchema(onDdfSchemaReady, isSilent = false) {
        const settings = lodash_1.cloneDeep(this.settings);
        settings.silent = isSilent;
        if (!settings.silent) {
            utils_1.logger.notice('loading generic content...');
        }
        const resources = this.getDataPackageResources().map(resource => {
            const simplifiedResource = lodash_1.cloneDeep(resource);
            if (!lodash_1.isArray(simplifiedResource.schema.primaryKey)) {
                simplifiedResource.schema.primaryKey = [simplifiedResource.schema.primaryKey];
            }
            simplifiedResource.schema.fields = simplifiedResource.schema.fields.map(field => field.name);
            return simplifiedResource;
        });
        const conceptsResources = resources.filter(resource => constants_1.getTypeByPrimaryKey(resource.schema.primaryKey) === constants_1.CONCEPT);
        const entitiesResources = resources.filter(resource => constants_1.getTypeByPrimaryKey(resource.schema.primaryKey) === constants_1.ENTITY);
        const getOrderedSection = (section) => {
            section.forEach(dataPointDescriptor => {
                dataPointDescriptor.resources = lodash_1.sortBy(dataPointDescriptor.resources);
            });
            return lodash_1.sortBy(section, ['primaryKey', 'value']);
        };
        if (!settings.silent) {
            utils_1.logger.notice('generating ddfSchema...');
        }
        ddf_schema_1.getDdfSchemaContent({
            resources,
            conceptsResources,
            entitiesResources,
            ddfDataSet: this,
            dataPackageDescriptor: this.dataPackageDescriptor
        }, (err, ddfSchema) => {
            if (err) {
                return onDdfSchemaReady(err);
            }
            ddfSchema.datapoints = getOrderedSection(ddfSchema.datapoints);
            ddfSchema.entities = getOrderedSection(ddfSchema.entities);
            ddfSchema.concepts = getOrderedSection(ddfSchema.concepts);
            ddfSchema.synonyms = getOrderedSection(ddfSchema.synonyms);
            onDdfSchemaReady(null, ddfSchema);
        });
    }
    writeDataPackage(settings, existingDataPackage, onDataPackageFileReady) {
        const dateLabel = new Date().toISOString().replace(/:/g, '');
        const isBasedOnCurrentDataPackage = (existingDataPackage && (settings.updateDataPackageTranslations || settings.updateDataPackageContent));
        const fileName = isBasedOnCurrentDataPackage || (!existingDataPackage || settings._newDataPackagePriority) ?
            data_package_1.DATA_PACKAGE_FILE : `${data_package_1.DATA_PACKAGE_FILE}.${dateLabel}`;
        const filePath = path_1.resolve(this.rootPath, fileName);
        const commandLineSettings = lodash_1.cloneDeep(this.settings);
        commandLineSettings.isProgressNeeded = true;
        this.createDataPackage(() => {
            const contentToOut = lodash_1.cloneDeep(isBasedOnCurrentDataPackage ? existingDataPackage :
                this.getDataPackageDescriptor().getDataPackageObject());
            if (settings.updateDataPackageTranslations) {
                contentToOut.translations = this.getDataPackageDescriptor().getDataPackageObject().translations;
            }
            if (settings.updateDataPackageContent) {
                contentToOut.resources = this.getDataPackageResources();
                contentToOut.ddfSchema = this.getDataPackageSchema();
            }
            if (!isBasedOnCurrentDataPackage) {
                contentToOut.ddfSchema = this.getDataPackageSchema();
            }
            const content = settings.compressDatapackage ?
                JSON.stringify(contentToOut) : JSON.stringify(contentToOut, null, 2);
            file_1.writeFile(filePath, content, err => onDataPackageFileReady(err, filePath));
        });
    }
}
exports.DdfDataSet = DdfDataSet;
