"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
const Levenshtein = require('levenshtein');
const SUGGEST_TOLERANCE = 3;
function getConceptHeaderDetails(ddfDataSet) {
    const result = [];
    ddfDataSet.getConcept().fileDescriptors.forEach(fileDescriptor => (fileDescriptor.headers || [])
        .filter(header => header !== constants_1.CONCEPT_ID && header !== constants_1.CONCEPT_TYPE)
        .forEach(header => result.push({ header, fileDescriptor })));
    return result;
}
function getDataPointHeaderDetails(ddfDataSet) {
    const result = [];
    ddfDataSet.getDataPoint().fileDescriptors.forEach(fileDescriptor => fileDescriptor.headers.forEach(header => result.push({ header, fileDescriptor })));
    return result;
}
function getEntityHeaderDetails(ddfDataSet) {
    const result = [];
    ddfDataSet.getEntity().fileDescriptors.forEach(fileDescriptor => {
        fileDescriptor.headers.forEach(header => {
            if (!ddf_things_1.looksLikeIsField(header)) {
                result.push({ header, fileDescriptor });
            }
        });
    });
    return result;
}
function getHeaderDetailObjects(ddfDataSet) {
    return getDataPointHeaderDetails(ddfDataSet)
        .concat(getEntityHeaderDetails(ddfDataSet))
        .concat(getConceptHeaderDetails(ddfDataSet));
}
function isNativeConcept(conceptName) {
    return conceptName === constants_1.CONCEPT_ID || conceptName === constants_1.CONCEPT_TYPE;
}
function setNonConceptHeaderIssue(conceptIds, detailObject, result) {
    const header = detailObject.header.replace(/^"/, '').replace(/"$/, '').replace(/^is--/, '');
    if (!lodash_1.includes(conceptIds, header) && !isNativeConcept(header)) {
        const suggestions = lodash_1.uniq(conceptIds
            .map(concept => {
            const levenshtein = new Levenshtein(concept, header);
            return {
                concept,
                distance: levenshtein.distance
            };
        })
            .filter(suggest => suggest.distance < SUGGEST_TOLERANCE)
            .map(suggest => suggest.concept));
        const issue = new issue_1.Issue(registry_1.NON_CONCEPT_HEADER)
            .setPath(detailObject.fileDescriptor.fullPath)
            .setData(header)
            .setSuggestions(suggestions);
        result.push(issue);
    }
}
exports.rule = {
    rule: (ddfDataSet) => {
        const result = [];
        const conceptIds = ddfDataSet.getConcept().getIds();
        getHeaderDetailObjects(ddfDataSet)
            .map(headerDetailObject => setNonConceptHeaderIssue(conceptIds, headerDetailObject, result));
        return result;
    }
};
