"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
exports.rule = {
    rule: (ddfDataSet) => {
        const resources = ddfDataSet.getDataPackageResources();
        const schema = ddfDataSet.getDataPackageSchema();
        if (!lodash_1.isObject(schema)) {
            return [new issue_1.Issue(registry_1.DATAPACKAGE_INCORRECT_PRIMARY_KEY).setPath(ddfDataSet.dataPackageDescriptor.rootFolder)
                    .setData({ reason: 'empty ddfSchema section' })];
        }
        const notPartOfFieldsReason = lodash_1.compact(lodash_1.flattenDeep(resources.map(resource => {
            const fields = resource.schema.fields.map(field => field.name).filter(field => !ddf_things_1.looksLikeIsField(field));
            const primaryKey = constants_1.getArrayInAnyCase(resource.schema.primaryKey);
            if (lodash_1.intersection(fields, primaryKey).length !== primaryKey.length) {
                return new issue_1.Issue(registry_1.DATAPACKAGE_INCORRECT_PRIMARY_KEY)
                    .setPath(ddfDataSet.dataPackageDescriptor.rootFolder)
                    .setData({ fields, primaryKey, reason: 'Primary key is not a part of fields' });
            }
            return null;
        })));
        const inconsistentPrimaryKeys = [];
        for (const resource of resources) {
            const primaryKey = lodash_1.isArray(resource.schema.primaryKey) ? resource.schema.primaryKey : [resource.schema.primaryKey];
            const pkType = constants_1.getTypeByPrimaryKey(primaryKey);
            if (!pkType) {
                inconsistentPrimaryKeys.push(new issue_1.Issue(registry_1.DATAPACKAGE_INCORRECT_PRIMARY_KEY)
                    .setPath(ddfDataSet.dataPackageDescriptor.rootFolder)
                    .setData({ primaryKey, reason: 'Unclassified primaryKey in resources' }));
            }
        }
        const schemaModes = [
            { type: constants_1.CONCEPT, section: 'concepts' },
            { type: constants_1.DATA_POINT, section: 'datapoints' },
            { type: constants_1.ENTITY, section: 'entities' },
            { type: constants_1.SYNONYM, section: 'synonyms' }
        ];
        for (const schemaMode of schemaModes) {
            const schemaData = schema[schemaMode.section] || [];
            for (const schemaRecord of schemaData) {
                const pkType = constants_1.getTypeByPrimaryKey(schemaRecord.primaryKey);
                if (pkType !== schemaMode.type) {
                    inconsistentPrimaryKeys.push(new issue_1.Issue(registry_1.DATAPACKAGE_INCORRECT_PRIMARY_KEY)
                        .setPath(ddfDataSet.dataPackageDescriptor.rootFolder)
                        .setData({ primaryKey: schemaRecord.primaryKey, reason: 'Unclassified primaryKey in ddfSchema' }));
                }
            }
        }
        return [...notPartOfFieldsReason, ...inconsistentPrimaryKeys];
    }
};
