"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const registry_1 = require("../../registry");
const issue_1 = require("../../issue");
const ddf_things_1 = require("../../../utils/ddf-things");
const constants_1 = require("../../../ddf-definitions/constants");
const isEntityDomainField = (conceptTypes, field) => conceptTypes[field] === ddf_things_1.CONCEPT_TYPE_ENTITY_DOMAIN;
const isEntitySetField = (conceptTypes, field) => conceptTypes[field] === ddf_things_1.CONCEPT_TYPE_ENTITY_SET;
class EntityFileDescriptor {
    constructor(entitySetFields, entityIdField) {
        this.entitySetFields = entitySetFields;
        this.entityIdField = entityIdField;
    }
    getRelatedEntities(record) {
        const relatedEntities = [];
        for (let entitySetField of this.entitySetFields) {
            if (ddf_things_1.isDdfTrue(record[entitySetField])) {
                relatedEntities.push(entitySetField.replace(/is--/, ''));
            }
        }
        return relatedEntities;
    }
}
class DuplicationDescriptor {
    constructor(relatedEntities, record) {
        this.relatedEntities = relatedEntities;
        this.record = record;
    }
}
class SearchUniqueEntitiesStory {
    constructor(ddfDataSet) {
        this.ddfDataSet = ddfDataSet;
        this.serviceData = {};
        this.dutyData = {};
    }
    fillServiceData() {
        this.serviceData.conceptTypeHash = this.ddfDataSet.getConcept().getDictionary(null, constants_1.CONCEPT_TYPE);
        this.serviceData.domainTypeHash = this.ddfDataSet.getConcept().getDictionary(null, constants_1.DOMAIN_ID);
        this.serviceData.entities = this.ddfDataSet.getEntity().getDataByFiles();
        this.serviceData.entityFiles = Object.keys(this.serviceData.entities);
        return this;
    }
    collectFieldDescriptors() {
        this.serviceData.entityFileDescriptors = {};
        for (let entityFile of this.serviceData.entityFiles) {
            const firstRecord = lodash_1.head(this.serviceData.entities[entityFile]);
            const allFields = Object.keys(firstRecord);
            const entitySetFields = [];
            const dataPackageResource = lodash_1.head(this.ddfDataSet.getDataPackageResources()
                .filter(resource => lodash_1.endsWith(entityFile, resource.path)));
            const entityIdField = dataPackageResource.schema.primaryKey;
            for (let field of allFields) {
                if (ddf_things_1.looksLikeIsField(field)) {
                    entitySetFields.push(field);
                }
            }
            this.serviceData.entityFileDescriptors[entityFile] = new EntityFileDescriptor(entitySetFields, entityIdField);
        }
        return this;
    }
    collectValueCounters() {
        this.dutyData.valueCounters = {};
        for (let entityFile of this.serviceData.entityFiles) {
            const entityFileDescriptor = this.serviceData.entityFileDescriptors[entityFile];
            for (let record of this.serviceData.entities[entityFile]) {
                const value = record[entityFileDescriptor.entityIdField];
                if (!this.dutyData.valueCounters[value]) {
                    this.dutyData.valueCounters[value] = 0;
                }
                this.dutyData.valueCounters[value]++;
            }
        }
        return this;
    }
    collectValueDuplicationDescriptors() {
        this.dutyData.duplicationDescriptors = {};
        for (let entityFile of this.serviceData.entityFiles) {
            const entityFileDescriptor = this.serviceData.entityFileDescriptors[entityFile];
            for (let record of this.serviceData.entities[entityFile]) {
                const value = record[entityFileDescriptor.entityIdField];
                if (this.dutyData.valueCounters[value] > 1) {
                    const relatedEntities = [];
                    if (isEntityDomainField(this.serviceData.conceptTypeHash, entityFileDescriptor.entityIdField)) {
                        relatedEntities.push(...entityFileDescriptor.getRelatedEntities(record));
                        relatedEntities.push(...this.getEntitySetFieldsForDomain(record, entityFileDescriptor.entityIdField));
                    }
                    if (isEntitySetField(this.serviceData.conceptTypeHash, entityFileDescriptor.entityIdField)) {
                        relatedEntities.push(entityFileDescriptor.entityIdField);
                    }
                    if (!this.dutyData.duplicationDescriptors[value]) {
                        this.dutyData.duplicationDescriptors[value] = [];
                    }
                    this.dutyData.duplicationDescriptors[value].push(new DuplicationDescriptor(relatedEntities, record));
                }
            }
        }
        return this;
    }
    countDuplicationsByEntitySets() {
        this.dutyData.entitiesCounters = {};
        const values = Object.keys(this.dutyData.duplicationDescriptors);
        for (let value of values) {
            const entitiesCounters = {};
            for (let duplicationDescriptor of this.dutyData.duplicationDescriptors[value]) {
                for (let relatedEntity of duplicationDescriptor.relatedEntities) {
                    if (!entitiesCounters[relatedEntity]) {
                        entitiesCounters[relatedEntity] = 0;
                    }
                    entitiesCounters[relatedEntity]++;
                }
            }
            this.dutyData.entitiesCounters[value] = entitiesCounters;
        }
        return this;
    }
    result() {
        const issues = [];
        const values = Object.keys(this.dutyData.duplicationDescriptors);
        const isDuplication = (value, relatedEntities) => {
            for (let relatedEntity of relatedEntities) {
                if (this.dutyData.entitiesCounters[value][relatedEntity] > 1) {
                    return true;
                }
            }
            return false;
        };
        for (let value of values) {
            let issue = null;
            for (let duplicationDescriptor of this.dutyData.duplicationDescriptors[value]) {
                if (isDuplication(value, duplicationDescriptor.relatedEntities)) {
                    if (!issue) {
                        issue = new issue_1.Issue(registry_1.NON_UNIQUE_ENTITY_VALUE).setData({ value, records: [] });
                    }
                    issue.data.records.push(duplicationDescriptor.record);
                }
            }
            issues.push(issue);
        }
        return lodash_1.compact(issues);
    }
    getEntitySetFieldsForDomain(firstRecord, entityIdField) {
        const entitySetFields = [];
        const allFields = Object.keys(firstRecord);
        let doesNotContainIsFields = true;
        for (let field of allFields) {
            if (ddf_things_1.looksLikeIsField(field)) {
                doesNotContainIsFields = false;
            }
        }
        if (doesNotContainIsFields) {
            for (let key of Object.keys(this.serviceData.domainTypeHash)) {
                if (entityIdField === this.serviceData.domainTypeHash[key]) {
                    entitySetFields.push(key);
                }
            }
        }
        return entitySetFields;
    }
}
exports.SearchUniqueEntitiesStory = SearchUniqueEntitiesStory;
