"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const Levenshtein = require("levenshtein");
const registry_1 = require("../registry");
const issue_1 = require("../issue");
const ddf_things_1 = require("../../utils/ddf-things");
const constants_1 = require("../../ddf-definitions/constants");
const SUGGEST_TOLERANCE = 5;
const getInformationAboutNonConcept = (ddfDataSet, actualHeaderDetail, headerDetail) => lodash_1.includes(ddfDataSet.getConcept().getIds(), actualHeaderDetail) ? null : {
    message: 'Not a concept',
    headerDetail
};
const getInformationAboutWrongNonEntityConcept = (ddfDataSet, actualHeaderDetail, headerDetail) => {
    const conceptRecord = ddfDataSet.getConcept().getRecordByKey(actualHeaderDetail);
    return !conceptRecord || conceptRecord.concept_type !== ddf_things_1.CONCEPT_TYPE_ENTITY_SET ? {
        message: 'Wrong concept type',
        headerDetail
    } : null;
};
const getInformationForbiddenDomain = (ddfDataSet, actualHeader, fileDescriptor) => {
    const domainTypeHash = ddfDataSet.getConcept().getDictionary(null, constants_1.DOMAIN_ID);
    const primaryKeyDomain = domainTypeHash[fileDescriptor.primaryKey] || fileDescriptor.primaryKey;
    const currentDomain = domainTypeHash[actualHeader];
    const isCurrentDomainExists = lodash_1.includes(ddfDataSet.getConcept().getAllData().map(record => record.concept), currentDomain);
    if (primaryKeyDomain === domainTypeHash[actualHeader] || !isCurrentDomainExists) {
        return null;
    }
    return {
        message: 'Forbidden domain for the entity set',
        currentEntitySet: actualHeader,
        currentDomain,
        primaryKeyDomain
    };
};
const getSuggestions = (ddfDataSet, actualHeader) => {
    return ddfDataSet.getConcept().getDataIdsByType(ddf_things_1.CONCEPT_TYPE_ENTITY_SET)
        .map(concept => {
        const levenshtein = new Levenshtein(concept, actualHeader);
        return {
            concept,
            distance: levenshtein.distance
        };
    })
        .filter(suggest => suggest.distance < SUGGEST_TOLERANCE)
        .map(suggest => suggest.concept);
};
exports.rule = {
    rule: (ddfDataSet) => {
        const result = [];
        ddfDataSet.getEntity().fileDescriptors.forEach(fileDescriptor => {
            fileDescriptor.headers.forEach(header => {
                let actualHeader = header;
                if (ddf_things_1.looksLikeIsField(header)) {
                    actualHeader = header.replace(ddf_things_1.fieldIsPrefix, '');
                    const data = getInformationAboutNonConcept(ddfDataSet, actualHeader, header) ||
                        getInformationAboutWrongNonEntityConcept(ddfDataSet, actualHeader, header) ||
                        getInformationForbiddenDomain(ddfDataSet, actualHeader, fileDescriptor);
                    if (data) {
                        const suggestions = getSuggestions(ddfDataSet, actualHeader);
                        const issue = new issue_1.Issue(registry_1.WRONG_ENTITY_IS_HEADER)
                            .setPath(fileDescriptor.fullPath)
                            .setData(data)
                            .setSuggestions(suggestions);
                        result.push(issue);
                    }
                }
            });
        });
        return result;
    }
};
