"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const events_1 = require("events");
const ddf_data_set_1 = require("./ddf-definitions/ddf-data-set");
const issues_filter_1 = require("./utils/issues-filter");
const shared_1 = require("./shared");
const utils_1 = require("./utils");
const fs = require("fs");
const data_package_1 = require("./data/data-package");
const ddf_rules_1 = require("./ddf-rules");
const lodash_1 = require("lodash");
const child_process = require('child_process');
const os = require('os');
const allCpuCount = os.cpus().length;
const resetGlobals = () => {
    Object.getOwnPropertySymbols(ddf_rules_1.allRules).forEach(dataPointRuleKey => {
        if (ddf_rules_1.allRules[dataPointRuleKey].resetStorage) {
            ddf_rules_1.allRules[dataPointRuleKey].resetStorage();
        }
    });
};
function mapToObject(map) {
    return Array.from(map.keys()).reduce((agg, key) => {
        agg[key] = map.get(key);
        return agg;
    }, {});
}
class ValidatorBase {
    constructor() {
        this.messageEmitter = new events_1.EventEmitter();
    }
    sendMessage(data) {
        this.messageEmitter.emit('message', data);
    }
    onMessage(data) {
        return this.messageEmitter.on('message', data);
    }
    abandon() {
        shared_1.supervisor.abandon = true;
        this.messageEmitter.emit('abandon', true);
    }
    isAbandoned() {
        return shared_1.supervisor.abandon;
    }
}
exports.ValidatorBase = ValidatorBase;
class JSONValidator extends ValidatorBase {
    constructor(rootPath, settings) {
        super();
        this.errorsSummaryByFileHash = new Map();
        this.errorsSummaryByRuleHash = new Map();
        this.warningsSummaryByFileHash = new Map();
        this.warningsSummaryByRuleHash = new Map();
        this.rootPath = rootPath;
        this.settings = settings || {};
        this.issueEmitter = new events_1.EventEmitter();
        this.issueEmitter.on('issue', issue => {
            this.fillSummary(issue);
        });
        this.issueEmitter.on('finish', error => {
            if (!error) {
                this.summary.errorsByFiles = mapToObject(this.errorsSummaryByFileHash);
                this.summary.errorsByRuleId = mapToObject(this.errorsSummaryByRuleHash);
                this.summary.warningsByFiles = mapToObject(this.warningsSummaryByFileHash);
                this.summary.warningsByRuleId = mapToObject(this.warningsSummaryByRuleHash);
            }
        });
    }
    on(type, data) {
        return this.issueEmitter.on(type, data);
    }
    validate() {
        this.initSummary();
        this.issuesFilter = new issues_filter_1.IssuesFilter(this.settings);
        this.ddfDataSet = new ddf_data_set_1.DdfDataSet(this.rootPath, this.settings);
        this.ddfDataSet.load(() => {
            shared_1.validationProcess(this, utils_1.logger, true);
        });
    }
    initSummary() {
        this.summary = {
            errors: 0,
            warnings: 0,
            errorsByFiles: {},
            errorsByRuleId: {},
            warningsByFiles: {},
            warningsByRuleId: {}
        };
        this.errorsSummaryByFileHash.clear();
        this.errorsSummaryByRuleHash.clear();
        this.warningsSummaryByFileHash.clear();
        this.warningsSummaryByRuleHash.clear();
    }
    fillSummary(issue) {
        const file = issue.path ? path.relative(this.rootPath, issue.path) : 'not related to file';
        if (issue.isWarning) {
            if (!this.errorsSummaryByFileHash.has[file]) {
                this.errorsSummaryByFileHash.set(file, 0);
            }
            this.errorsSummaryByFileHash.set(file, this.errorsSummaryByFileHash.get(file) + 1);
            if (!this.errorsSummaryByRuleHash.has(issue.id)) {
                this.errorsSummaryByRuleHash.set(issue.id, 0);
            }
            this.errorsSummaryByRuleHash.set(issue.id, this.errorsSummaryByRuleHash.get(issue) + 1);
            this.summary.errors++;
        }
        else {
            if (!this.warningsSummaryByFileHash.has(file)) {
                this.warningsSummaryByFileHash.set(file, 0);
            }
            this.warningsSummaryByFileHash.set(file, this.warningsSummaryByFileHash.get(file) + 1);
            if (!this.warningsSummaryByRuleHash.has(issue.id)) {
                this.warningsSummaryByRuleHash.set(issue.id, 0);
            }
            this.warningsSummaryByRuleHash.set(issue.id, this.warningsSummaryByRuleHash.get(issue.id) + 1);
            this.summary.warnings++;
        }
    }
}
exports.JSONValidator = JSONValidator;
class StreamValidator extends ValidatorBase {
    constructor(rootPath, settings) {
        super();
        this.rootPath = rootPath;
        this.settings = settings || {};
        this.issueEmitter = new events_1.EventEmitter();
        utils_1.getTransport().updateSettings(this.settings);
    }
    on(type, data) {
        return this.issueEmitter.on(type, data);
    }
    multiThreadProcessing() {
        const cpuCount = allCpuCount - (this.settings.useAllCpu ? 0 : 1);
        const filesChunks = shared_1.getDataPointFilesChunks(this.ddfDataSet, cpuCount);
        const total = filesChunks.reduce((result, chunk) => result + chunk.length, 0);
        const childProcesses = [];
        utils_1.logger.progressInit('datapoints validation', { total });
        let childProcessesFinished = 0;
        for (let index = 0; index < cpuCount; index++) {
            const childProcess = child_process.fork(path.resolve(this.settings.appPath || __dirname, 'thread.js'));
            childProcess.on('message', (message) => {
                if (message.finish) {
                    childProcessesFinished++;
                }
                if (message.progress) {
                    utils_1.logger.progress();
                }
                if (!message.finish && message.issue) {
                    this.issueEmitter.emit('issue', message.issue);
                }
                if (message.finish && childProcessesFinished === cpuCount) {
                    this.issueEmitter.emit('finish', message.err);
                }
            });
            childProcess.send({
                rootPath: this.rootPath,
                settings: this.settings,
                filesChunks: filesChunks[index],
                isCollectResultMode: false
            });
            childProcesses.push(childProcess);
        }
        this.messageEmitter.on('abandon', () => {
            for (const childProcess of childProcesses) {
                try {
                    childProcess.send('abandon');
                }
                catch (e) {
                }
            }
        });
    }
    validate() {
        this.sendMessage('loading dataset...');
        this.issuesFilter = new issues_filter_1.IssuesFilter(this.settings);
        this.ddfDataSet = new ddf_data_set_1.DdfDataSet(this.rootPath, this.settings);
        this.ddfDataSet.load(() => {
            shared_1.validationProcess(this, utils_1.logger);
        });
    }
}
exports.StreamValidator = StreamValidator;
class SimpleValidator extends ValidatorBase {
    constructor(rootPath, settings) {
        super();
        this.rootPath = rootPath;
        this.settings = settings || {};
        this.issueEmitter = new events_1.EventEmitter();
        this.isDataSetCorrect = true;
        if (!this.settings.excludeTags) {
            this.settings.excludeTags = '';
        }
        this.settings.excludeTags += ' WARNING ';
        this.settings.silent = true;
        utils_1.getTransport().updateSettings(this.settings);
    }
    on(type, data) {
        return this.issueEmitter.on(type, data);
    }
    validate() {
        this.issuesFilter = new issues_filter_1.IssuesFilter(this.settings);
        this.ddfDataSet = new ddf_data_set_1.DdfDataSet(this.rootPath, this.settings);
        this.ddfDataSet.load(() => {
            shared_1.simpleValidationProcess(this);
        });
    }
}
exports.SimpleValidator = SimpleValidator;
function getDataPackageInfo(ddfRootFolder) {
    const ddfPath = path.resolve(ddfRootFolder || '.');
    const dataPackagePath = path.resolve(ddfPath, data_package_1.DATA_PACKAGE_FILE);
    return { ddfPath, dataPackagePath, exists: fs.existsSync(dataPackagePath) };
}
exports.getDataPackageInfo = getDataPackageInfo;
function createDataPackage(parameters, onNotice, onDataPackageReady) {
    const expectedSettings = parameters.externalSettings || utils_1.settings;
    let { ddfPath, dataPackagePath, exists } = getDataPackageInfo(parameters.ddfRootFolder);
    let newDataPackagePath;
    if (parameters.newDataPackagePriority && exists) {
        const dateLabel = new Date().toISOString().replace(/:/g, '');
        const newFileName = `${data_package_1.DATA_PACKAGE_FILE}.${dateLabel}`;
        newDataPackagePath = path.resolve(ddfPath, newFileName);
        fs.renameSync(dataPackagePath, newDataPackagePath);
    }
    let dataPackageContent = null;
    if (exists) {
        try {
            dataPackageContent = JSON.parse(fs.readFileSync(newDataPackagePath || dataPackagePath, 'utf-8'));
        }
        catch (err) {
            return onDataPackageReady(`datapackage.json error: ${err}.`);
        }
    }
    onNotice('datapackage creation started...');
    const ddfDataSet = new ddf_data_set_1.DdfDataSet(ddfPath, { silent: true });
    ddfDataSet.createDataPackage(() => {
        onNotice('resources are ready');
        expectedSettings._newDataPackagePriority = parameters.newDataPackagePriority;
        ddfDataSet.writeDataPackage(expectedSettings, dataPackageContent, (err, filePath) => {
            if (err) {
                return onDataPackageReady(`datapackage.json was NOT created: ${err}.`);
            }
            onNotice(`${filePath} was created successfully.`);
            onDataPackageReady();
        });
    });
}
exports.createDataPackage = createDataPackage;
function getDataPackageActuality(parameters, cb) {
    let { ddfPath, dataPackagePath, exists } = getDataPackageInfo(parameters.ddfRootFolder);
    if (!exists) {
        return cb('datapackage does not exist!');
    }
    let dataPackageContent = null;
    try {
        dataPackageContent = JSON.parse(fs.readFileSync(dataPackagePath, 'utf-8'));
    }
    catch (err) {
        cb(err.toString());
    }
    const dataPackage = new data_package_1.DataPackage(ddfPath, { silent: true });
    dataPackage.take(() => {
        const newDataPackageContent = dataPackage.getDataPackageObject();
        const generatedResources = newDataPackageContent.resources.map(r => data_package_1.stripDdfPrefix(r.name));
        const currentResources = dataPackageContent.resources.map(r => data_package_1.stripDdfPrefix(r.name));
        const diff = lodash_1.difference(generatedResources, currentResources);
        const newResources = diff.filter(r => lodash_1.includes(generatedResources, r));
        if (!lodash_1.isEmpty(newResources)) {
            cb(`datapackage is not actual: more details:\n${JSON.stringify(newResources, null, 2)}`);
        }
        else {
            cb(`\ndatapackage is actual\n`);
        }
    }, true);
}
exports.getDataPackageActuality = getDataPackageActuality;
exports.validate = validator => {
    shared_1.supervisor.abandon = false;
    resetGlobals();
    validator.validate();
};
