"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const ddf_rules_1 = require("./ddf-rules");
const data_point_chunks_processing_1 = require("./stories/data-point-chunks-processing");
const registry_1 = require("./ddf-rules/registry");
const constants_1 = require("./ddf-definitions/constants");
exports.supervisor = {
    abandon: false
};
const getValidationSyncResultBySimpleRules = (ddfDataSet, rulesKeys) => {
    const allIssuesSources = [];
    for (let key of rulesKeys) {
        if (exports.supervisor.abandon) {
            return null;
        }
        const issuesSources = ddf_rules_1.allRules[key].rule(ddfDataSet);
        if (!lodash_1.isEmpty(issuesSources)) {
            if (lodash_1.isArray(issuesSources)) {
                allIssuesSources.push(...issuesSources);
            }
            if (!lodash_1.isArray(issuesSources)) {
                allIssuesSources.push(issuesSources);
            }
        }
    }
    return allIssuesSources;
};
const injectTranslations = (fileDescriptors) => {
    const translationDescriptors = [];
    for (let fileDescriptor of fileDescriptors) {
        translationDescriptors.push(...fileDescriptor.getExistingTranslationDescriptors());
    }
    fileDescriptors.push(...translationDescriptors);
};
exports.CONCURRENT_OPERATIONS_AMOUNT = 2;
exports.sameTranslation = (key, fileDescriptor) => fileDescriptor.isTranslation === ddf_rules_1.allRules[key].isTranslation;
exports.noTranslation = (key, fileDescriptor) => !fileDescriptor.isTranslation && !ddf_rules_1.allRules[key].isTranslation;
exports.isSimpleRule = ruleObject => !!ruleObject.rule;
exports.getDataPointFileDescriptorsGroups = (ddfDataSet, fileDescriptors) => {
    const dataPointsGroups = {};
    const resources = ddfDataSet.getDataPackageResources();
    const fileDescriptorsHash = fileDescriptors.reduce((hash, fileDescriptor) => {
        hash[fileDescriptor.file] = fileDescriptor;
        return hash;
    }, {});
    for (let resource of resources) {
        if (constants_1.getTypeByPrimaryKey(resource.schema.primaryKey) === constants_1.DATA_POINT) {
            const fields = resource.schema.fields.map(field => {
                let constraint = '';
                if (field.constraints && field.constraints.enum) {
                    constraint = '@' + field.constraints.enum.sort().join('-');
                }
                return field.name + constraint;
            });
            const key = fields.sort().join('--');
            if (!dataPointsGroups[key]) {
                dataPointsGroups[key] = [];
            }
            dataPointsGroups[key].push(fileDescriptorsHash[resource.path]);
        }
    }
    return lodash_1.compact(lodash_1.values(dataPointsGroups));
};
exports.getAllDataPointFileDescriptorsChunks = (ddfDataSet) => {
    const chunks = [];
    const descriptorsGroupsWithSize = exports.getDataPointFileDescriptorsGroups(ddfDataSet, ddfDataSet.getDataPoint().fileDescriptors)
        .map(descriptorsGroup => {
        const size = descriptorsGroup.reduce((sum, fileDescriptor) => sum + fileDescriptor.size, 0);
        return { descriptorsGroup, size };
    });
    const descriptorsGroupsSortedBySize = lodash_1.sortBy(descriptorsGroupsWithSize, ['size']);
    for (let descriptorPosition = 0; descriptorPosition < descriptorsGroupsSortedBySize.length; descriptorPosition++) {
        const expectedDescriptorsGroup = descriptorsGroupsSortedBySize[descriptorPosition].descriptorsGroup;
        injectTranslations(expectedDescriptorsGroup);
        chunks.push(expectedDescriptorsGroup);
    }
    return chunks;
};
exports.getDataPointFileDescriptorsChunks = (ddfDataSet, cpuCount) => {
    const chunks = [];
    const descriptorsGroupsWithSize = exports.getDataPointFileDescriptorsGroups(ddfDataSet, ddfDataSet.getDataPoint().fileDescriptors)
        .map(descriptorsGroup => {
        const size = descriptorsGroup.reduce((sum, fileDescriptor) => sum + fileDescriptor.size, 0);
        return { descriptorsGroup, size };
    });
    const descriptorsGroupsSortedBySize = lodash_1.sortBy(descriptorsGroupsWithSize, ['size']);
    for (let index = 0; index < cpuCount; index++) {
        chunks.push([]);
    }
    for (let index = 0; index < descriptorsGroupsSortedBySize.length; index += cpuCount) {
        for (let completeIndex = 0, descriptorPosition = index + completeIndex; completeIndex < cpuCount && descriptorPosition < descriptorsGroupsSortedBySize.length; completeIndex++, descriptorPosition = index + completeIndex) {
            const expectedDescriptorsGroup = descriptorsGroupsSortedBySize[descriptorPosition].descriptorsGroup;
            injectTranslations(expectedDescriptorsGroup);
            chunks[completeIndex].push(expectedDescriptorsGroup);
        }
    }
    return chunks;
};
exports.getDataPointFilesChunks = (ddfDataSet, cpuCount) => {
    const dataPointFileDescriptorsChunks = exports.getDataPointFileDescriptorsChunks(ddfDataSet, cpuCount);
    return dataPointFileDescriptorsChunks.map(chunkForProcess => chunkForProcess.map(dataPointFileDescriptors => dataPointFileDescriptors.map(dataPointFileDescriptor => dataPointFileDescriptor.fullPath)));
};
exports.getSimpleRulesResult = (ddfDataSet, issuesFilter) => {
    const rulesKeys = Object.getOwnPropertySymbols(ddf_rules_1.allRules).filter(key => exports.isSimpleRule(ddf_rules_1.allRules[key]) && issuesFilter.isAllowed(key));
    return getValidationSyncResultBySimpleRules(ddfDataSet, rulesKeys);
};
exports.getPreResult = (ddfDataSet, issuesFilter) => {
    const rulesKeys = Object.getOwnPropertySymbols(ddf_rules_1.allRules)
        .filter(key => lodash_1.includes(registry_1.tags[key], registry_1.DATAPACKAGE_TAG) && issuesFilter.isAllowed(key));
    return getValidationSyncResultBySimpleRules(ddfDataSet, rulesKeys);
};
exports.validationProcess = (context, logger, isCollectResultMode) => {
    context.sendMessage('validating concepts and entities...');
    const preRulesResult = exports.getPreResult(context.ddfDataSet, context.issuesFilter);
    if (!lodash_1.isEmpty(preRulesResult)) {
        const preIssuesToOut = [];
        preRulesResult.forEach((issue) => {
            preIssuesToOut.push(issue.view());
            context.issueEmitter.emit('issue', issue.view());
        });
        context.issueEmitter.emit('finish', null, preIssuesToOut);
    }
    else {
        const simpleRulesResult = exports.getSimpleRulesResult(context.ddfDataSet, context.issuesFilter);
        const allIssuesToOut = [];
        if (exports.supervisor.abandon) {
            context.issueEmitter.emit('finish');
            return;
        }
        if (!lodash_1.isEmpty(simpleRulesResult)) {
            simpleRulesResult.forEach((issue) => {
                allIssuesToOut.push(issue.view());
                context.issueEmitter.emit('issue', issue.view());
            });
        }
        if (context.settings.datapointlessMode) {
            context.issueEmitter.emit('finish', null, allIssuesToOut);
        }
        if (!context.settings.datapointlessMode && (context.settings.isMultithread && context.multiThreadProcessing)) {
            context.multiThreadProcessing();
        }
        if (!context.settings.datapointlessMode && (!context.settings.isMultithread || !context.multiThreadProcessing)) {
            const fileDescriptorsChunks = exports.getAllDataPointFileDescriptorsChunks(context.ddfDataSet);
            const dataPointChunksProcessingStory = new data_point_chunks_processing_1.DataPointChunksProcessingStory(fileDescriptorsChunks, context.issueEmitter);
            const theEnd = (out = []) => {
                context.issueEmitter.emit('finish', null, allIssuesToOut.concat(out));
            };
            context.issueEmitter.on('init-chunk-progress', (total) => {
                context.sendMessage('validating datapoints...');
                logger.progressInit('datapoints validation', { total });
            });
            context.issueEmitter.on('chunk-progress', () => {
                if (exports.supervisor.abandon) {
                    context.issueEmitter.emit('finish');
                    return;
                }
                logger.progress();
            });
            dataPointChunksProcessingStory.waitForResult(theEnd, isCollectResultMode).processDataPointChunks(context.ddfDataSet, context.issuesFilter);
        }
    }
};
exports.simpleValidationProcess = (context) => {
    const preRulesResult = exports.getPreResult(context.ddfDataSet, context.issuesFilter);
    if (!lodash_1.isEmpty(preRulesResult)) {
        context.issueEmitter.emit('finish', null, false);
    }
    else {
        const simpleRulesResult = exports.getSimpleRulesResult(context.ddfDataSet, context.issuesFilter);
        let isDataSetCorrect = true;
        if (!lodash_1.isEmpty(simpleRulesResult)) {
            context.issueEmitter.emit('finish', null, false);
        }
        else {
            if (context.settings.datapointlessMode) {
                context.issueEmitter.emit('finish', null, isDataSetCorrect);
            }
            if (!context.settings.datapointlessMode) {
                const fileDescriptorsChunks = exports.getAllDataPointFileDescriptorsChunks(context.ddfDataSet);
                const dataPointChunksProcessingStory = new data_point_chunks_processing_1.DataPointChunksProcessingStory(fileDescriptorsChunks, context.issueEmitter);
                const theEnd = (out = []) => {
                    context.issueEmitter.emit('finish', null, isDataSetCorrect && lodash_1.isEmpty(out));
                };
                dataPointChunksProcessingStory.waitForResult(theEnd, true).processDataPointChunks(context.ddfDataSet, context.issuesFilter);
            }
        }
    }
};
