"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const lodash_1 = require("lodash");
const yargs = require("yargs");
const shared_1 = require("../data/shared");
const myName = 'validate-ddf';
const argv = yargs
    .usage(`Usage: ${myName} [root] [options]`)
    .command('root', 'DDF Root directory')
    .example(`${myName} ../ddf-example`, 'validate DDF datasets for the root')
    .example(`${myName} ../ddf-example -i`, 'generate datapackage.json file')
    .example(`${myName} ../ddf-example --summary`, 'show summary data regarding the issues after validation')
    .example(`${myName} ../ddf-example -i --translations`, 'update only "translations" section in datapackage.json')
    .example(`${myName} ../ddf-example -i --translations --content`, 'rewrite "translations", "resources" and "ddfSchema" sections in datapackage.json')
    .example(`${myName} ../ddf-example -a`, 'check if datapackage.json in ../ddf-example folder actual')
    .example(`${myName} ../ddf-example -j`, 'fix JSONs for this DDF dataset')
    .example(`${myName} --rules`, 'print information regarding supported rules')
    .example(`${myName} ../ddf-example --ws`, 'apply Waffle Server specific rules. See WAFFLE_SERVER_TAG tag based rules in the rules list (--rules flag)')
    .example(`${myName} ../ddf-example --multithread`, 'validate datapoints for `ddf-example` in separate threads')
    .example(`${myName} ../ddf-example --multithread --use-all-cpu`, 'use all CPU during validation via multithread mode')
    .example(`${myName} ../ddf-example --hidden`, 'allow hidden folders validation')
    .example(`${myName} ../ddf-example --include-rules "INCORRECT_JSON_FIELD"`, 'Validate only by INCORRECT_JSON_FIELD rule')
    .example(`${myName} ../ddf-example --exclude-tags "WARNING_TAG"`, 'Get all kinds of issues except warnings')
    .example(`${myName} ../ddf-example --exclude-dirs "etl,foo-dir"`, 'validate "ddf-example" and its subdirectories except "etl" and "foo-dir"')
    .example(`${myName} ../ddf-example --exclude-dirs "'dir1 with spaces','dir2 with spaces'"`, 'validate "ddf-example" and its subdirectories that contain spaces')
    .example(`${myName} ../ddf-example --exclude-dirs '"dir1 with spaces","dir2 with spaces"'`, 'validate "ddf-example" and its subdirectories that contain spaces: case 2')
    .example(`${myName} ../ddf-example -i --compress-datapackage --heap 4096`, 'Create compressed datapackage.json via 4Gb heap')
    .describe('i', 'Generate datapackage.json file')
    .describe('a', 'Check datapackage.json file actuality')
    .describe('compress-datapackage', 'Compress datapackage.json file')
    .describe('translations', 'Rewrite "translations" section in existing datapackage.json')
    .describe('content', 'Rewrite "resources" and "ddfSchema" sections in existing datapackage.json')
    .describe('summary', 'Show summary data after validation in case of errors are found')
    .describe('j', 'Fix wrong JSONs')
    .describe('rules', 'print information regarding supported rules')
    .describe('multithread', 'validate datapoints in separate threads')
    .describe('use-all-cpu', 'use all CPU during validation via multithread mode')
    .describe('datapointless', 'forget about datapoint validation')
    .describe('silent', `don't show progress of validation and print issues to the screen`)
    .describe('hidden', 'allow hidden folders validation')
    .describe('ws', 'apply Waffle Server specific rules')
    .describe('include-tags', 'Process only issues by selected tags')
    .describe('exclude-tags', 'Process all tags except selected')
    .describe('include-rules', 'Process only issues by selected rules')
    .describe('exclude-rules', 'Process all rules except selected')
    .describe('exclude-dirs', 'Process all directories except selected. Directories should be separated via "," character')
    .describe('heap', `Set custom heap size:
1024 will increase heap to 1gb
2048 will increase heap to 2gb
3072 will increase heap to 3gb
4096 will increase heap to 4gb
5120 will increase heap to 5gb
6144 will increase heap to 6gb
7168 will increase heap to 7gb
8192 will increase heap to 8gb`)
    .argv;
exports.getDDFRootFolder = () => lodash_1.head(argv._) || process.cwd();
exports.getSettings = () => {
    const settings = {};
    const options = ['include-tags', 'exclude-tags', 'include-rules', 'exclude-rules', 'exclude-dirs', 'heap'];
    const setMiscSettings = () => {
        settings.isDataPackageGenerationMode = !!argv.i;
        settings.isDataPackageActual = !!argv.a;
        settings.isJsonAutoCorrectionMode = !!argv.j;
        settings.versionShouldBePrinted = !!argv.v;
        settings.datapointlessMode = !!argv.datapointless;
        settings.silent = !!argv.silent;
        settings.updateDataPackageTranslations = !!argv.translations;
        settings.updateDataPackageContent = !!argv.content;
        settings.isSummaryNeeded = !!argv.summary;
        settings.isPrintRules = !!argv.rules;
        settings.isCheckHidden = !!argv.hidden;
        settings.isMultithread = !!argv.multithread;
        settings.isWaffleServer = !!argv.ws;
        settings.useAllCpu = !!argv['use-all-cpu'];
        settings.compressDatapackage = argv['compress-datapackage'];
    };
    setMiscSettings();
    options.forEach(option => {
        settings[lodash_1.camelCase(option)] = argv[option];
        if (option === 'exclude-tags' && !settings.isWaffleServer) {
            settings[lodash_1.camelCase(option)] += ' WAFFLE_SERVER_TAG';
        }
    });
    settings.excludeDirs = shared_1.getExcludedDirs(settings);
    return settings;
};
