"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const fs_1 = require("fs");
const shared_1 = require("../data/shared");
const utils_1 = require("../utils");
const stripBom = require('strip-bom');
const csv = require('fast-csv');
const END_OF_LINE = require('os').EOL;
const homeFolder = process.platform === 'win32' ? process.env.USERPROFILE : process.env.HOME;
function copyFile(source, target, onFileCopied) {
    const rd = fs_1.createReadStream(source);
    const wr = fs_1.createWriteStream(target);
    let cbCalled = false;
    rd.on('error', err => done(err));
    wr.on('error', err => done(err));
    wr.on('close', done);
    rd.pipe(wr);
    function done(err) {
        if (!cbCalled) {
            onFileCopied(err);
            cbCalled = true;
        }
    }
}
function norm(folder) {
    let normFolder = folder;
    if (folder.indexOf('~') !== -1) {
        normFolder = path_1.join(homeFolder, folder.replace('~', ''));
    }
    return path_1.resolve(normFolder);
}
exports.norm = norm;
function readDir(_dir, done) {
    const dir = norm(_dir);
    fs_1.readdir(dir, (err, list) => {
        if (err) {
            return done(err);
        }
        done(null, list);
    });
}
exports.readDir = readDir;
function walkDir(_dir, done) {
    const dir = norm(_dir);
    let results = [];
    fs_1.readdir(dir, (err, list) => {
        if (err || !shared_1.isPathExpected(dir)) {
            return done(err, []);
        }
        let pending = list.length;
        if (!pending) {
            return done(null, results);
        }
        list.forEach(file => {
            file = path_1.resolve(dir, file);
            fs_1.stat(file, (err, stat) => {
                if (stat && stat.isDirectory()) {
                    results.push(file);
                    walkDir(file, (err, res) => {
                        results = results.concat(res);
                        if (!--pending) {
                            done(null, results);
                        }
                    });
                }
                if (!stat || !stat.isDirectory()) {
                    if (!--pending) {
                        done(null, results);
                    }
                }
            });
        });
    });
}
exports.walkDir = walkDir;
function writeFile(path, content, onFileWrote) {
    fs_1.writeFile(path, content, err => {
        if (err) {
            return onFileWrote(err);
        }
        onFileWrote();
    });
}
exports.writeFile = writeFile;
function getFileLine(filename, lineNo, callback) {
    const options = {
        flags: 'r',
        encoding: 'utf-8',
        fd: null,
        bufferSize: 64 * 1024
    };
    const stream = fs_1.createReadStream(filename, options);
    let fileData = '';
    stream.on('data', data => {
        fileData += stripBom(data);
        const lines = fileData.split(END_OF_LINE);
        if (lines.length >= +lineNo) {
            stream.destroy();
            stream.close();
            callback(null, lines[+lineNo]);
        }
    });
    stream.on('error', err => {
        callback(err, null);
    });
    stream.on('end', () => {
        stream.close();
        callback('File end reached without finding line', null);
    });
}
exports.getFileLine = getFileLine;
function readFile(filePath, onFileRead) {
    const content = [];
    csv
        .fromPath(filePath, { headers: true })
        .on('data', ddfRecord => content.push(ddfRecord))
        .on('end', () => {
        onFileRead(null, content);
    });
}
exports.readFile = readFile;
function walkFile(filePath, onLineRead, onFileRead) {
    fileExists(filePath, (err, exists) => {
        if (err) {
            utils_1.logger.error(err);
        }
        if (err || !exists) {
            return onFileRead();
        }
        let line = 0;
        csv
            .fromPath(filePath, { headers: true })
            .on('data', ddfRecord => onLineRead(ddfRecord, line++))
            .on('end', () => {
            onFileRead();
        })
            .on('error', err => {
            utils_1.logger.error(err);
        });
    });
}
exports.walkFile = walkFile;
function backupFile(filePath, onBackupCreated) {
    fs_1.stat(filePath, (statErr, stats) => {
        if (statErr) {
            throw statErr;
        }
        if (stats.isFile()) {
            copyFile(filePath, filePath + '.backup', fileErr => {
                if (fileErr) {
                    throw fileErr;
                }
                onBackupCreated();
            });
        }
    });
}
exports.backupFile = backupFile;
function fileExists(file, onFileChecked) {
    fs_1.stat(file, (err, stats) => {
        if (err) {
            if (err.code === 'ENOENT') {
                return onFileChecked(null, false);
            }
            return onFileChecked(err);
        }
        onFileChecked(null, stats.isFile());
    });
}
exports.fileExists = fileExists;
